

export interface SessionNodeInfo {
	name: string
	type: number
	shape: number[]
}

export type DataTypeString = "float32" | "float64" | "float" | "double" | "int32" | "uint32" | "int16" | "uint16" | "int8" | "uint8" | "int64" | "uint64"

export enum DataType {
	Unknown,
	Float32,
	Float64,
	Int32,
	Uint32,
	Int16,
	Uint16,
	Int8,
	Uint8,
	Int64,
	Uint64,
}

export type SessionNodeData = Float32Array | Float64Array | Int32Array | Uint32Array | Int16Array | Uint16Array | Int8Array | Uint8Array | BigInt64Array | BigUint64Array

export interface SessionRunInputOption {
	type?: DataTypeString
	data: SessionNodeData
	shape?: number[]
}

export abstract class CommonSession {
	public abstract run(inputs: Record<string, SessionNodeData | SessionRunInputOption>): Promise<Record<string, Float32Array>>

	public abstract get inputs(): Record<string, SessionNodeInfo>;
	public abstract get outputs(): Record<string, SessionNodeInfo>;
}

export function isTypedArray(val: any): val is SessionNodeData {
	return val?.buffer instanceof ArrayBuffer;
}

export function dataTypeFrom(str: DataTypeString) {
	return {
		"float32": DataType.Float32,
		"float64": DataType.Float64,
		"float": DataType.Float32,
		"double": DataType.Float64,
		"int32": DataType.Int32,
		"uint32": DataType.Uint32,
		"int16": DataType.Int16,
		"uint16": DataType.Uint16,
		"int8": DataType.Int8,
		"uint8": DataType.Uint8,
		"int64": DataType.Int64,
		"uint64": DataType.Uint64,
	}[str] ?? DataType.Unknown;
}

export function dataTypeToString(type: DataType): DataTypeString | null {
	switch (type) {
		case DataType.Float32: return "float32";
		case DataType.Float64: return "float64";
		case DataType.Int32: return "int32";
		case DataType.Uint32: return "uint32";
		case DataType.Int16: return "int16";
		case DataType.Uint16: return "uint16";
		case DataType.Int8: return "int8";
		case DataType.Uint8: return "uint8";
		case DataType.Int64: return "int64";
		case DataType.Uint64: return "uint64";
		default: return null;
	}
}