export interface FacePoint {
	x: number
	y: number
}

type PointType = "leftEye" | "rightEye" | "leftEyebrow" | "rightEyebrow" | "nose" | "mouth" | "contour"

export abstract class FaceAlignmentResult {
	#points: FacePoint[]

	public constructor(points: FacePoint[]) { this.#points = points; }

	/** 关键点 */
	public get points() { return this.#points; }

	/** 获取特定的关键点 */
	public getPointsOf(type: PointType | PointType[]) {
		if (typeof type == "string") type = [type];
		const result: FacePoint[] = [];
		for (const t of type) {
			for (const idx of this[`${t}PointIndex` as const]()) {
				result.push(this.points[idx]);
			}
		}
		return result;
	}

	/** 方向 */
	public get direction() {
		const [{ x: x1, y: y1 }, { x: x2, y: y2 }] = this.directionPointIndex().map(idx => this.points[idx]);
		return Math.atan2(y1 - y2, x2 - x1)
	}

	/** 用于判断方向的两个点的索引(建议选取眼球中间的点) */
	protected abstract directionPointIndex(): [number, number];
	/** 左眼点的索引 */
	protected abstract leftEyePointIndex(): number[];
	/** 右眼点的索引 */
	protected abstract rightEyePointIndex(): number[];
	/** 左眉点的索引 */
	protected abstract leftEyebrowPointIndex(): number[];
	/** 右眉点的索引 */
	protected abstract rightEyebrowPointIndex(): number[];
	/** 嘴巴点的索引 */
	protected abstract mouthPointIndex(): number[];
	/** 鼻子的索引 */
	protected abstract nosePointIndex(): number[];
	/** 轮廓点的索引 */
	protected abstract contourPointIndex(): number[];

	protected indexFromTo(from: number, to: number) {
		const indexes: number[] = [];
		for (let i = from; i <= to; i++) {
			indexes.push(i);
		}
		return indexes;
	}
}
