import { cv } from "../../cv";
import { ImageCropOption, ImageSource, Model } from "../common/model";

export interface FaceRecognitionPredictOption extends ImageCropOption { }

export abstract class FaceRecognition extends Model {
	public abstract doPredict(image: cv.Mat, option?: FaceRecognitionPredictOption): Promise<number[]>;

	public async predict(image: ImageSource, option?: FaceRecognitionPredictOption) { return Model.resolveImage(image, im => this.doPredict(im, option)); }
}

export function cosineDistance(lhs: number[], rhs: number[]) {
	if (lhs.length !== rhs.length) throw new Error('length not match');

	const getMod = (vec: number[]) => {
		let sum = 0;
		for (let i = 0; i < vec.length; ++i) sum += vec[i] * vec[i];
		return sum ** 0.5;
	}

	let temp = 0;
	for (let i = 0; i < lhs.length; ++i) temp += lhs[i] * rhs[i];
	return temp / (getMod(lhs) * getMod(rhs));
}

export function euclideanDistance(lhs: number[], rhs: number[]) {
	if (lhs.length !== rhs.length) throw new Error('length not match');
	let sumDescriptor = 0;
	for (let i = 0; i < lhs.length; i++) sumDescriptor += (lhs[i] - rhs[i]) ** 2;
	return sumDescriptor ** 0.5;
}
